#!/bin/bash
### BEGIN INIT INFO
# Provides:          lightning-init.sh
# Required-Start:    networking
# Required-Stop:     networking
# Should-Start:      sshd tor
# Should-Stop:       sshd tor
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: lightningd init script
# Description:       This file should be used to construct scripts to be
#                    placed in /etc/init.d.
### END INIT INFO

PATH=/sbin:/usr/sbin:/bin:/usr/bin
DESC="Init lightningd"
NAME=lightning-init.sh
DAEMON=/usr/lib/lightning-init/bin/lightningd
LIGHTNING_CLI=/usr/lib/lightning-init/bin/lightning-cli
DAEMON_ARGS=""
SCRIPTNAME=/etc/init.d/$(/usr/bin/basename $0)
STATIC_DEFAULT_CONF=/usr/lib/lightning-init/default/lightning-init
DEFAULT_CONF=/etc/default/lightning-init

. /lib/init/vars.sh
. /lib/lsb/init-functions

# Exit if the package is not installed
[ -x "$DAEMON" ] || exit 0

if [ -f ${STATIC_DEFAULT_CONF} ]
then
    . ${STATIC_DEFAULT_CONF}
fi

if [ -f ${DEFAULT_CONF} ]
then
    . ${DEFAULT_CONF}
fi

set -e

LOG_DIR=/var/log
RUN_DIR=`/usr/bin/getent passwd "${RUNUSER}" | /usr/bin/cut -d: -f6`

LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} ).log
TESTNET_LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-testnet.log

LOG_FILE_DEBUG=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-debug.log
TESTNET_LOG_FILE_DEBUG=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-testnet-debug.log

if [ -z "${RUN_VERSION}" ]
then
    RUN_VERSION=testnet
fi

if ! [ "${ENABLE}" = "yes" ]
then
    log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
    exit 0
fi

#
# Function that starts the daemon/service
#
do_start()
{
    INIT_TYPE="$1"
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	/bin/echo -n "" >> ${LOG_FILE}
	/bin/echo -n "" >> ${TESTNET_LOG_FILE}
	/bin/echo -n "" >> ${LOG_FILE_DEBUG}
	/bin/echo -n "" >> ${TESTNET_LOG_FILE_DEBUG}

	/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE_DEBUG}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE_DEBUG}

	/bin/chmod 0640 ${LOG_FILE}
	/bin/chmod 0640 ${TESTNET_LOG_FILE}
	/bin/chmod 0640 ${LOG_FILE_DEBUG}
	/bin/chmod 0640 ${TESTNET_LOG_FILE_DEBUG}

	case "${RUN_VERSION}" in
	    main)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} --conf=${SERVER_CONF} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
	    ;;
	    testnet)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} --conf=${TESTNET_SERVER_CONF} 1>/dev/null 2>>${TESTNET_LOG_FILE}"
		RESULT=$?
	    ;;
	    both)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} --conf=${SERVER_CONF} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} --conf=${TESTNET_SERVER_CONF} 1>/dev/null 2>>${TESTNET_LOG_FILE}"
		RESULT=$[ ${RESULT} + $? ]
	    ;;
	esac
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that stops the daemon/service
#
do_stop()
{
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	case "${RUN_VERSION}" in
	    main)
		PIDO=`/bin/cat ${RUN_DIR}/lightningd.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LIGHTNING_CLI} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
	    ;;
	    testnet)
		TESTNET_PIDO=`/bin/cat ${RUN_DIR}/lightningd-testnet.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LIGHTNING_CLI} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
	    ;;
	    both)
		PIDO=`/bin/cat ${RUN_DIR}/lightningd.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LIGHTNING_CLI} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		TESTNET_PIDO=`/bin/cat ${RUN_DIR}/lightningd-testnet.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LIGHTNING_CLI} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$[ ${RESULT} + $? ]
	    ;;
	esac

	COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	COUNTER=0
	while [ ${COUNT_PROC} -gt 0 ]
	do
	    COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	    COUNTER=$[ ${COUNTER} + 1 ]
	    if [ ${COUNTER} -ge 95 ]
	    then
		COUNT_PROC=0
	    fi
	    /bin/sleep 1
	done
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that sends a SIGHUP to the daemon/service
#
do_reload()
{
    do_stop
    /bin/sleep 1
    do_start
}

RESULT=0

case "$1" in
  start)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start lightnind"
	do_start
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  stop)
	log_daemon_msg "${NAME}"
	log_progress_msg "Stop lightnind"
	do_stop
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  reload)
	log_daemon_msg "${NAME}"
	log_progress_msg "Reload lightnind"
	do_reload
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  restart)
	log_daemon_msg "${NAME}"
	log_progress_msg "Restart lightnind"
	do_reload
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  *)
	/bin/echo "Usage: $SCRIPTNAME {start|stop|restart|reload}" >&2
	exit 3
	;;
esac

exit ${RESULT}
